package UploadImageResizer::CMS;

use strict;

sub add_input {
    my ($cb, $app, $tmpl) = @_;

    my $old;

    if (MT->version_number >= 5.1) {
        $old = <<HERE;
  <mt:include name="include/category_selector.tmpl">
  </mt:if>
HERE
    } elsif (MT->version_number >= 5.0) {
        $old = <<HERE;
        <mt:include name="include/category_selector.tmpl">
    </mt:if>
HERE
    } else {
        return;
    }

    $old = quotemeta($old);

    my $plugin = MT->component("UploadImageResizer");
    my $message_option = $plugin->translate("Option");
    my $message_width = $plugin->translate("Width");
    my $message_height = $plugin->translate("Height");
    my $message_none = $plugin->translate("None");
    my $message_square = $plugin->translate("Square");
    my $message_aspect = $plugin->translate("Keep aspect");
    my $message_overwrite = $plugin->translate("Overwrite");
    my $hint1 = $plugin->translate("hint1");
    my $hint2 = $plugin->translate("hint2");
    my $hint3 = $plugin->translate("hint3");

    my $new = <<HERE;
  <mt:include name="include/category_selector.tmpl">
  </mt:if>

  <mtapp:setting
     id="upload_option"
     label_class="none">
    <input id="upload_option" name="upload_option" type="checkbox" value="upload_option" /> <label for="upload_option">$message_option</label>
  </mtapp:setting>

<div id="option" style="display:none">
  <mtapp:setting
     id="width"
     label_class="top-label"
     label="$message_width"
     hint="$hint1"
     show_hint="1">
    <input id="width" name="width" type="text" class="text" style="width:100px" />
  </mtapp:setting>

  <mtapp:setting
     id="height"
     label_class="top-label"
     label="$message_height"
     hint="$hint1"
     show_hint="1">
    <input id="height" name="height"  type="text" class="text" style="width:100px" />
  </mtapp:setting>

  <mtapp:setting
     id="image_option0"
     label_class="none">
    <input id="image_option0" name="image_option" type="radio" value="none" /> <label for="image_option0">$message_none</label>
  </mtapp:setting>

  <mtapp:setting
     id="image_option1"
     label_class="none"
     show_hint="1"
     hint="$hint2">
    <input id="image_option1" name="image_option" type="radio" value="aspect" /> <label for="image_option1">$message_aspect</label>
  </mtapp:setting>

  <mtapp:setting
     id="image_option2"
     label_class="none"
     show_hint="1"
     hint="$hint3">
    <input id="image_option2" name="image_option"  type="radio" value="square" /> <label for="image_option2">$message_square</label>
  </mtapp:setting>

<mt:ignore>
  <mtapp:setting
     id="overwrite_yes"
     label_class="none">
    <input id="overwrite_yes" name="overwrite_yes" type="checkbox" value="1" /> <label for="overwrite_yes">$message_overwrite</label>
  </mtapp:setting>
</mt:ignore>
</div>
<script>
jQuery('#upload_option').change(function(){
    if (jQuery(this).attr('checked')) {
        jQuery('#option').css('display','block');
    } else {
        jQuery('#option').css('display','none');
    }
});
</script>
HERE
    $$tmpl =~ s/$old/$new/;
}

sub resize_image {
    my $cb = shift;
    my (%param) = @_;

    return if MT->version_number < 5.0;

    my $app = MT->instance;

    my $width = $app->param('width');
    my $height = $app->param('height');
    return if ( !$width && !$height );

    return if !$app->param('upload_option');
    my $option = $app->param('image_option');

    my $fmgr;
    my $asset = $param{asset};
    my $blog = $param{Blog} || $asset->blog;

    require MT::FileMgr;
    $fmgr ||= $blog ? $blog->file_mgr : MT::FileMgr->new('Local');
    return undef unless $fmgr;

    my $file_path = $asset->file_path;
    return undef unless $fmgr->exists($file_path);

    my ( $i_h, $i_w ) = ( $asset->image_height, $asset->image_width );
    return undef unless $i_h && $i_w;
    return if ( $i_h < $height || $i_w < $width );

    if ( $option eq 'square' ) {
        if ( $width && !$height ) {
            $height = $width;
        } elsif ( !$width && $height ) {
            $width = $height;
        } else {
            if ( $width > $height ) {
                $width = $height;
            } else {
                $height = $width;
            }
        }

        require MT::Image;
        my %square
            = MT::Image->inscribe_square( Width => $width, Height => $height );
        ( $i_h, $i_w ) = @square{qw( Size Size )};
    }

    # find the longest dimension of the image:
    my ( $n_h, $n_w );
    if ( $option eq 'aspect' ) {
       ( $n_h, $n_w ) = _get_dimension( $i_h, $i_w, $height, $width );
    } elsif ( $option eq 'none' || $option eq 'square' ) {
        $n_h = $height;
        $n_w = $width;
        if ( !$n_h || !$n_w ) {
            ( $n_h, $n_w ) = _get_dimension( $i_h, $i_w, $height, $width );
        }
    } elsif ( $option eq '' ) {
        return;
    }

    my $data;
    if (   ( $n_w == $i_w )
        && ( $n_h == $i_h )
        && !$param{Square}
        && !$param{Type} )
    {
        $data = $fmgr->get_data( $file_path, 'upload' );
    }
    else {

        # create a thumbnail for this file
        require MT::Image;
        my $img = new MT::Image( Filename => $file_path )
            or return $asset->error( MT::Image->errstr );

        if ( $option eq 'square' ) {
            ($data) = $img->make_square()
                or return $asset->error(
                MT->translate( "Error cropping image: [_1]", $img->errstr ) );
        }

        ($data) = $img->scale( Height => $n_h, Width => $n_w )
            or return $asset->error(
            MT->translate( "Error scaling image: [_1]", $img->errstr ) );

#        if ( my $type = $param{Type} ) {
#            ($data) = $img->convert( Type => $type )
#                or return $asset->error(
#                MT->translate( "Error converting image: [_1]", $img->errstr )
#                );
#        }
    }

    $fmgr->put_data( $data, $file_path, 'upload' )
        or return $asset->error(
        MT->translate( "Error creating thumbnail file: [_1]", $fmgr->errstr )
        );

    my $asset = $param{asset};
    $asset->image_width($n_w);
    $asset->image_height($n_h);
    $asset->save;
}

sub _get_dimension {
    my ( $i_h, $i_w, $h, $w ) = @_;

    my ( $n_h, $n_w ) = ( $i_h, $i_w );
    my $scale = '';
    if ( $h && !$w ) {
        $scale = 'h';
    }
    elsif ( $w && !$h ) {
        $scale = 'w';
    }
    else {
        if ( $i_h > $i_w ) {

            # scale, if necessary, by height
            if ( $i_h > $h ) {
                $scale = 'h';
            }
            elsif ( $i_w > $w ) {
                $scale = 'w';
            }
        }
        else {

            # scale, if necessary, by width
            if ( $i_w > $w ) {
                $scale = 'w';
            }
            elsif ( $i_h > $h ) {
                $scale = 'h';
            }
        }
    }
    if ( $scale eq 'h' ) {

        # scale by height
        $n_h = $h;
        $n_w = int( $i_w * $h / $i_h );
    }
    elsif ( $scale eq 'w' ) {

        # scale by width
        $n_w = $w;
        $n_h = int( $i_h * $w / $i_w );
    }
    return ( $n_h, $n_w );
}

1;
